def fn load_levels% (file$)
rem import a level file
rem return nbr of levels if ok, -1 if not
rem . = ground (empty)
rem # = wall
rem 1-9 atoms
rem run through the file twice
local chan%,lp%,a$,maxl%,maxcol%,lwidth%,lheight%,must_rotate%,temp%,levels%
local lines%,cols%,b$
  chan%=fop_in(file$)
  if chan%<0:return -1
  levels%=-1
  rep lp%
    if eof(#chan%): exit lp%
    a$=gline$(#chan%)		: rem get one line
    temp%= "[Level" instr a$
    if temp%
      if "[levelset" instr a$:next lp%
      temp%=extract_number%(a$,"[Level","]")
      if temp%>levels%:levels%=temp%
      next lp%
    endif
  end rep lp%
  if levels%=-1
    close#chan%
    return -1
  endif
  get#chan%\0
  dim grids%(levels%,max_lines%,max_cols%) : rem the grid is displayed line by line
  dim names$(levels%,40)	: rem names of levels
  dim atoms%(levels%,36)	: rem the atoms in each level : type of atom
  dim atom_bounds%(levels%,36,8): rem  max of 8 bounds per atom each element = the typr of bond or -1 if none
  dim walls%(levels%,15,2)	: rem this is where the first wall is in each line
				: rem if either dim is -1, this is an empty line
  dim moles%(levels%,15,15)
  dim mole_lines%(levels%)	: rem nbr of lines in each mole
  dim mole_lengths%(levels%)
  rep lp%
    if eof(#chan%): exit lp%
    a$=gline$(#chan%)		: rem get one line
    a$=strip_spaces$(a$)
    if a$="":next lp%
    if "[Level" instr a$  and not( "[levelset" instr a$)
      get_level_data chan%,a$
    endif
  end rep lp%
  close#chan%
  level$=levelprim$&level%&")"
  return levels%
end def load_levels
:
def proc get_level_data (chan%, lev$)
rem gets data on one level
rem chan%= (local) from caller
local lp%,b$,level%,line%,col%,a$,t%
  level%=extract_number%(lev$,"[Level","]")
  if level%=-1:return		: rem ooops????
  rep lp%
     if eof(#chan%): exit lp%
     a$=gline$(#chan%)
     a$=strip_spaces$(a$)
     if a$="":exit lp%		       : rem level ends with a newline
     t%="feld" instr a$
     if t%
	extract_field a$,t%
	next lp%
     endif
     t%="atom" instr a$
     if t%
	extract_atom a$,t%
	next lp%
     endif
     t%= "mole" instr a$
     if t%
	extract_mole a$,t%
	next lp%
     endif
     t%= "name=" instr a$
     if t% and len(a$)>5
	names$(level%)=a$(t%+5 to)
	next lp%
     endif
   end rep lp%
end def get_level_data
:
def proc extract_atom (a$,place%)
local temp%,b$,atm%,a_typ%,lp%
rem level% is local from caller
rem line i  file is "atom_1=1-c"
rem where "atom_1" is the atom nbr as it appears in the grid
rem	  the number/letter after the "=" is the type of atom (1 = hydrogen etc)
rem	  -c is the bound ("a" is top=north, then clockwise to g = north west)
rem	  bounds A to D = double bounds, E to G =triple (no bounds in corners for these)
  atm%=extract_one_number%(a$,"atom_")	: rem which atom in grid
  rem if atm%=-1:return 		    : rem ?????
  if atm%=-1:atm%=warning%("1",mnch%,a$):return 		    : rem ?????
  a_typ%="=" instr a$
  if not a_typ%:atm%=warning%("2",mnch%,a$):return
  if not a_typ%:return
  a_typ%=code(a$(a_typ%+1))
  select on a_typ%
    = 48 to 57 : a_typ%=a_typ%-48	: rem 0-9 -> 0...9
    = 65 to 68 : a_typ%=a_typ%-42	: rem A-D -> 23+
    = 69 to 77 : a_typ%=a_typ%-58	: rem E-M -> 10...20
    = 111 : a_typ%=22			: rem "o"
    = remainder:atm%=warning%("3 ->"&a_typ%,mnch%,a$):return
  end select
  atoms%(level%,atm%)=a_typ%		: rem atom in grid = this type
  temp%="-" instr a$
  if not temp%
      b$=""
  else
      b$=a$(temp%+1 to) 		: rem b$=bounds
  endif
  for lp%=1 to len(b$)
    temp%=code(b$(lp%))
    if temp%>75
      temp%=temp%-97			: rem a...g ->	0...7
    else
      temp%=temp%-57			: rem A...H ->	8...15
    endif
    atom_bounds%(level%,atm%,lp%)=temp%
  end for lp%
  for lp%=len(b$)+1 to 8: atom_bounds%(level%,atm%,lp%)=-1
end def extract_atom
:
def proc extract_field(a$)
local line%,b$,temp%,lp%,f%,l%
  line%=extract_number%(a$,"feld_","=")  : rem line in grid
  if line%=-1:return
  t%="=" instr a$
  if t%=-1:return
  if len(a$)<> t%+15:return
  t%=t%+1
  f%=-1
  l%=-1
  for lp%=0 to 14
     b$=a$(t%+lp%)
     if b$==".":next lp%:exit lp%
     if f%=-1:f%=lp%	: rem the first square that isn't empty%
     l%=lp%		: rem last square that isn't empty
     if b$=="#":grids%(level%,line%,lp%)=-1:next lp%:exit lp%
     temp%=code(b$)
     if temp%<60
	temp%=temp%-48 : rem 1-9 becomes 1-9
     else
	temp%=temp%-87 : rem a... becomes 10...
     endif
     rem add sanity check here
     grids%(level%,line%,lp%)=temp%
  end for lp%
  walls%(level%,line%,0)=f%   : rem first wall
  walls%(level%,line%,1)=l%   : rem last wall
end def extract_field
:
def proc extract_mole (a$)
local temp%,mmline%,lp%,b$
  mmline%=extract_number%(a$,"mole_","=")  : rem nbr of file in mole
  if mmline%=-1:return
  if mmline%>mole_lines%(level%):mole_lines%(level%)=mmline%
  t%="=" instr a$
  if t%=-1 or t%=len(a$):return
  t%=t%+1
  mole_lengths%(level%)=len(a$)-t%
  for lp%=t% to len(a$)
     b$=a$(lp%)
     temp%=code(b$)
     select on temp%
       =48 to 57  : temp%= temp%-48
       =97 to 122 : temp%= temp%-87
       =remainder : next lp%:exit lp%
     end select
    moles%(level%,mmline%,lp%-t%)=temp%
  end for lp%
end def extract_mole
:
def proc read_level
local lpy%,lpx%,cols%
dim grid%(max_lines%,max_cols%)
  for lpy%=0 to max_lines%
     for lpx%=0 to max_cols%
	grid%(lpy%,lpx%)=grids%(current_level%,lpy%,lpx%)
     end for lpx%
  end for lpy%
  dim mole%(15,15)
  for lpy%=0 to mole_lines%(current_level%)
     for cols%=0 to max_cols%
	mole%(lpy%,cols%)=moles%(current_level%,lpy%,cols%)
     end for cols%
  end for lpy%
  mole_length%=mole_lengths%(current_level%)
end def read_level
:
